#!/bin/bash
#
#
# Filename: BUGGER_Autoinstall_retrieve.sh
#
# Retrieve Auto install information for a project
#
#
# Copyright Ben Tasker 2010
# Released under the GNU GPL V3
# See http://benscomputer.no-ip.org/LICENSE3 for more info.


# Config

# Grab our config
./BUGGER_config.sh imascript /tmp/BUGGER.front_page_process.conf.$$

source /tmp/BUGGER.front_page_process.conf.$$
rm -f /tmp/BUGGER.front_page_process.conf.$$ 2> /dev/null



# No authentication needed, will have been called by a remote script

Autoupdate_function ()
{
# The remote client will need 
#
# Version
# Tarball location
# Install Script Location
# Test version?
# Checksum of tarball


# This will be based on the information the client provides to us. 
# Client should provide
#
# It's current Version number
# Project ID
# Allow Unstable updates?


# To find appropriate updates we need to query
#
# Project ID and Old version number
#
# We can currently only query one column at a time.
# Field most likely to return fewest possible matches is
# version number, so we'll query that, then check each result
# for a matching project ID.




PROJ_ID=$( /bin/env | grep "REQUEST_URI" | sed -n 's/^.*PROJ=\([^&]*\).*$/\1/p' )
CLIENT_VERSION=$( /bin/env | grep "REQUEST_URI" | sed -n 's/^.*CURRVERSION=\([^&]*\).*$/\1/p' )
RETURN_TESTING=$( /bin/env | grep "REQUEST_URI" | sed -n 's/^.*TESTING=\([^&]*\).*$/\1/p' )


wget -q -O /tmp/.BUGGER.AutoUpdate.$$ "http://$DATABASE_SERVER/cgi-bin/Claims_DB_listener.sh?USER=$DBUSERNAME&PASS=$DBPASS&METHOD=4&DBNAME=Bugs&TABLE=AutoUpdate&QCOL=10&QSTRING=$CLIENT_VERSION"

# Check for ERROR or NORESULTS

STATUS=$( cat /tmp/.BUGGER.AutoUpdate.$$ )

if [ "$STATUS" == "ERROR" ]
then

/bin/cat << EOM
ERROR: An Unspecified Claims_DB Error Occurred
EOM

rm -f /tmp/.BUGGER.AutoUpdate.$$ 2> /dev/null
exit

fi


if [ "$STATUS" == "NORESULTS" ]
then


/bin/cat << EOM
ERROR: An Invalid current version was specified - No matching records found
EOM


rm -f /tmp/.BUGGER.AutoUpdate.$$ 2> /dev/null
exit


fi



# No we've checked for errors, we need to start processing the results

KEEP_IN_LOOP="Y"


while read -r a
do

/bin/cat << EOM | sed s/\"//g > /tmp/.BUGGER.AutoUpdate.lbl.$$
$a
EOM

PROJECT_ID=$( cat /tmp/.BUGGER.AutoUpdate.lbl.$$ | awk -F\, '{print $2}' )

# See if the project ID matches the one the client gave us, if not we aren't interested!!!
if [ "$PROJECT_ID" == "$PROJ_ID" ]
then

VERSION=$( cat /tmp/.BUGGER.AutoUpdate.lbl.$$ | awk -F\, '{print $3}' )
TARBALL_LOCAT=$( cat /tmp/.BUGGER.AutoUpdate.lbl.$$ | awk -F\, '{print $5}' )
SCRIPT_LOCATION=$( cat /tmp/.BUGGER.AutoUpdate.lbl.$$ | awk -F\, '{print $4}' )
TESTING_VERSION=$( cat /tmp/.BUGGER.AutoUpdate.lbl.$$ | awk -F\, '{print $6}' )
TARBALL_CHECKSUM=$( cat /tmp/.BUGGER.AutoUpdate.lbl.$$ | awk -F\, '{print $9}' )

# Check whether we should be returning unstable updates.
if [ "$RETURN_TESTING" == "Y" ]
then
if [ "$TESTING_VERSION" == "Y" ]
then

OUTPUTRESULTS="Y"
else
OUTPUTRESULTS="N"
fi
else
if [ "$TESTING_VERSION" != "Y" ]
then
OUTPUTRESULTS="Y"
else
OUTPUTRESULTS="N"
fi

fi


if [ "$OUTPUTRESULTS" == "Y" ]
then
# Output the information

/bin/cat << EOM

TARBALL_LOCATION="$TARBALL_LOCAT"
INSTALL_SCRIPT="$SCRIPT_LOCATION"
TESTING_VERSION="$TESTING_VERSION"
TARBALL_CHECKSUM="$TARBALL_CHECKSUM"
UPDATE_VERSION="$VERSION"

EOM
fi

fi

done < /tmp/.BUGGER.AutoUpdate.$$

rm -f /tmp/.BUGGER.AutoUpdate.lbl.$$ 2> /dev/null
rm -f /tmp/.BUGGER.AutoUpdate.$$ 2> /dev/null

exit

}




autoinstall_information_request ()
{
# The remote client will need
#
# - Tarball location
# - Install Script Location
# - Allow Custom Install Scripts?
# - Version
# - Tarball Checksum
# - Project License


# In order to retrieve this information, we are expecting the following information
#
# - Project ID
# - Return testing information



PROJ_ID=$( /bin/env | grep "REQUEST_URI" | sed -n 's/^.*PROJ=\([^&]*\).*$/\1/p' )

RETURN_TESTING=$( /bin/env | grep "REQUEST_URI" | sed -n 's/^.*TESTING=\([^&]*\).*$/\1/p' )


wget -q -O /tmp/.BUGGER.Autoinstall.$$ "http://$DATABASE_SERVER/cgi-bin/Claims_DB_listener.sh?USER=$DBUSERNAME&PASS=$DBPASS&METHOD=4&DBNAME=Bugs&TABLE=AutoInstall&QCOL=2&QSTRING=$PROJ_ID"

# Check for either ERROR or NORESULTS in the response

STATUS=$( cat /tmp/.BUGGER.Autoinstall.$$ )

if [ "$STATUS" == "ERROR" ]
then
/bin/cat << EOM
ERROR: An Unspecified Claims_DB Error Occurred
EOM

rm -f /tmp/.BUGGER.Autoinstall.$$ 2> /dev/null
exit
fi

if [ "$STATUS" == "NORESULTS" ]
then
/bin/cat << EOM
ERROR: An Invalid Project ID was specified - No matching records found
EOM


rm -f /tmp/.BUGGER.Autoinstall.$$ 2> /dev/null
exit
fi


while read -r a
do

/bin/cat << EOM | sed s/\"//g > /tmp/.BUGGER.Autoinstall.lbl.$$
$a
EOM


TARBALL_LOCAT=$( cat /tmp/.BUGGER.Autoinstall.lbl.$$ | awk -F\, '{print $3}'
INSTALL_SCRIPT_LOCAT=$( cat /tmp/.BUGGER.Autoinstall.lbl.$$ | awk -F\, '{print $4}' )
ALLOW_CUSTOM_INSTALL_SCRIPT=$( cat /tmp/.BUGGER.Autoinstall.lbl.$$ | awk -F\, '{print $5}' )
TESTING_INDICATOR=$( cat /tmp/.BUGGER.Autoinstall.lbl.$$ | awk -F\, '{print $9}' )
TARBALL_CHECKSUM=$( cat /tmp/.BUGGER.Autoinstall.lbl.$$ | awk -F\, '{print $10}' )
VERSION=$( cat /tmp/.BUGGER.Autoinstall.lbl.$$ | awk -F\, '{print $6}' )
PROJECT_LICENSE=$( cat /tmp/.BUGGER.Autoinstall.lbl.$$ | awk -F\, '{print $11}' )

if [ "$RETURN_TESTING" == "Y" ]
then

if [ "$TESTING_INDICATOR" == "Y" ]
then
/bin/cat << EOM

TARBALL_LOCATION="$TARBALL_LOCAT"
INSTALL_SCRIPT="$INSTALL_SCRIPT_LOCAT"
ALLOW_CUSTOM_INSTALLS="$ALLOW_CUSTOM_INSTALL_SCRIPT"
TESTING_VERSION="$TESTING_INDICATOR"
TARBALL_CHECKSUM="$TARBALL_CHECKSUM"
PROJECT_VERSION="$VERSION"
LICENSE="$PROJECT_LICENSE"

EOM

fi

else


if [ "$TESTING_INDICATOR" != "Y" ]
then
/bin/cat << EOM

TARBALL_LOCATION="$TARBALL_LOCAT"
INSTALL_SCRIPT="$INSTALL_SCRIPT_LOCAT"
ALLOW_CUSTOM_INSTALLS="$ALLOW_CUSTOM_INSTALL_SCRIPT"
TESTING_VERSION="$TESTING_INDICATOR"
TARBALL_CHECKSUM="$TARBALL_CHECKSUM"
PROJECT_VERSION="$VERSION"
LICENSE="$PROJECT_LICENSE"

EOM

fi

fi



done < /tmp/.BUGGER.Autoinstall.$$







rm -f /tmp/.BUGGER.Autoinstall.lbl.$$ 2> /dev/null
rm -f /tmp/.BUGGER.Autoinstall.$$ 2> /dev/null

exit
}



# Function Main
#
#

# Output our MIME Type

echo Content-Type: text/plain
echo


# Will need to process whether to check updates or install once updates implemented
REQUEST_TYPE=$( /bin/env | grep "REQUEST_URI" | sed -n 's/^.*TYPE=\([^&]*\).*$/\1/p' )

# REQUEST_TYPE should either be
#
# Install
# Update
#
#


# Call the autoinstall function
if [ "$REQUEST_TYPE" == "Install" ]
then
autoinstall_information_request
fi

if [ "$REQUEST_TYPE" == "Update" ]
then
Autoupdate_function
fi


# We don't recognise the value of REQUEST_TYPE
# Possibly because it wasn't supplied

/bin/cat << EOM
ERROR: Unknown value passed for TYPE. (You supplied $REQUEST_TYPE)
EOM

exit